@@ -914,6 +914,11 @@ impl Connection {
         }?;
         Ok(self)
     }
+
+    /// Allows the quic library to check if session tickets are expected
+    pub fn are_session_tickets_enabled(&self) -> bool {
+        unsafe { s2n_connection_are_session_tickets_enabled(self.connection.as_ptr()) }
+    }
 }
 
 impl AsRef<Connection> for Connection {
