@@ -596,9 +596,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     }
 
     /* And set the signature and hash algorithm used for key exchange signatures */
-    POSIX_GUARD(s2n_choose_sig_scheme_from_peer_preference_list(conn,
-            &conn->handshake_params.client_sig_hash_algs,
-            &conn->handshake_params.conn_sig_scheme));
+    POSIX_GUARD_RESULT(s2n_signature_algorithm_select(conn));
 
     /* And finally, set the certs specified by the final auth + sig_alg combo. */
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
@@ -833,7 +831,7 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_conn_find_name_matching_certs(conn));
 
     POSIX_GUARD(s2n_set_cipher_as_sslv2_server(conn, client_hello->cipher_suites.data, client_hello->cipher_suites.size / S2N_SSLv2_CIPHER_SUITE_LEN));
-    POSIX_GUARD(s2n_choose_default_sig_scheme(conn, &conn->handshake_params.conn_sig_scheme, S2N_SERVER));
+    POSIX_GUARD_RESULT(s2n_signature_algorithm_select(conn));
     POSIX_GUARD(s2n_select_certs_for_server_auth(conn, &conn->handshake_params.our_chain_and_key));
 
     S2N_ERROR_IF(session_id_length > s2n_stuffer_data_available(in), S2N_ERR_BAD_MESSAGE);
